import { /* Http, I18n, */ Http, IEventParam, Metadata /* , IEventParam */ } from '@zeedhi/core';
import { Modal, ModalService } from '@zeedhi/common';
import { ECharts } from '@zeedhi/zd-echarts-common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

interface OptionalDataChart {
  IDCONFRECEBIMENTO?: string;
  CDOPERADOR?: string;
  REGIAO?: string;
  ESTADO?: string;
  NMESTADO?: string;
}

interface DataChart extends OptionalDataChart {
  LABEL: string;
  VALUE: string | number;
}

interface DataResponse extends OptionalDataChart {
  name: string;
  value: string | number;
}

export class AFGeneralController {
  private gGeneralAFModal!: Modal;
  
  public transformResponseInDataChart(response: DataChart[]) {
    const data = response.map((resp) => {
      const dataResponse: DataResponse = {
        name: resp.LABEL,
        value: resp.VALUE,
      };
      
      if (resp.IDCONFRECEBIMENTO !== undefined) {
        dataResponse.IDCONFRECEBIMENTO = resp.IDCONFRECEBIMENTO;
      }
      
      if (resp.CDOPERADOR !== undefined) {
        dataResponse.CDOPERADOR = resp.CDOPERADOR;
      }
      
      if (resp.REGIAO !== undefined) {
        dataResponse.REGIAO = resp.REGIAO;
      }
      
      if (resp.NMESTADO !== undefined) {
        dataResponse.ESTADO = resp.NMESTADO;
      }
      
      return dataResponse;
    });
    return data;
  }
  
  public getAFGeneralChartData({ component }: IEventParam<ECharts>) {
    const props = component.options.props;
    this.createModal();
    const routeName = props.routeName;
    Http.get(`/${routeName}?requestType:FilterData`).then(({ data }) => {
      const response = data.dataset.getAFGeneral;
      
      const dataToChart = this.transformResponseInDataChart(response);
      // component.options.series.data = dataToChart;
      const options = {
        series: {
          data: [
            ...dataToChart,
          ],
        },
      };
      component.updateChart(options);
    });
  }
  
  public async createModal() {
    this.gGeneralAFModal = await ModalService.createFromJson('/modals/getAFGeneralTAGridModal', true);
  }
  
  public async callModal(seriesData) {
    await this.gGeneralAFModal.show();
    const gridGroup = Metadata.getInstance<TekGrid>('getAFGeneralTAGridGroup');
    gridGroup.title = seriesData.name;
    // (gridGroup.datasource as typeof gridGroup.datasource & { route: string }).route = 'getAFGeneralTAGrid';
    gridGroup.datasource.setFilter({
      OPERADOR: seriesData.CDOPERADOR,
      IDCONFRECEBIMENTO: seriesData.IDCONFRECEBIMENTO,
      NMESTADO: seriesData.ESTADO,
      NMREGIAOGEO: seriesData.REGIAO,
    });
  }
  
  public drillDown({ component, param }: IEventParam<ECharts>) {
    const drillLevel = component.drillHistory.length;
    const seriesData = param.data;
    
    if (drillLevel >= 3) {
      return this.callModal(seriesData);
    }
    
    let queryString = `requestType=FilterData&level=${drillLevel}`;
    if (seriesData.IDCONFRECEBIMENTO !== undefined) {
      queryString += `&IDCONFRECEBIMENTO=${seriesData.IDCONFRECEBIMENTO}`;
    }
    
    if (seriesData.CDOPERADOR !== undefined) {
      queryString += `&operador=${seriesData.CDOPERADOR}`;
    }
    
    if (seriesData.REGIAO !== undefined) {
      queryString += `&regiao=${seriesData.REGIAO}`;
    }
    
    return Http.get(`/drillDownAFGeneral?${queryString}`).then((response) => {
      const dataToChart = this.transformResponseInDataChart(response.data.dataset.drillDownAFGeneral);
      let title = '';
      
      if (drillLevel === 0) {
        title = seriesData.name;
      } else {
        title += `>${seriesData.name}`;
      }
      
      const newOptions = {
        title: {
          text: title,
        },
        series: {
          type: 'pie',
          data: [
            ...dataToChart,
          ],
        },
      };
      component.drillDown(newOptions);
    });
  }
  
  public drillUp() {
  }
}
